#if LOGCAT_SERVER

using System;
using System.Collections.Generic;
using UnityEditor.IMGUI.Controls;
using UnityEngine;

namespace UnityEditor.TreeViewExamples
{
	internal class IPLogTreeView : TreeViewWithTreeModel<LogTreeElement>
	{
		const float kRowHeights = 20f;
		const float kToggleWidth = 18f;
		public bool showControls = true;

		// All columns
		enum MyColumns
		{
			IP,
			STATE
		}

		public static void TreeToList (TreeViewItem root, IList<TreeViewItem> result)
		{
			if (root == null)
				throw new NullReferenceException("root");
			if (result == null)
				throw new NullReferenceException("result");

			result.Clear();
	
			if (root.children == null)
				return;

			Stack<TreeViewItem> stack = new Stack<TreeViewItem>();
			for (int i = root.children.Count - 1; i >= 0; i--)
				stack.Push(root.children[i]);

			while (stack.Count > 0)
			{
				TreeViewItem current = stack.Pop();
				result.Add(current);

				if (current.hasChildren && current.children[0] != null)
				{
					for (int i = current.children.Count - 1; i >= 0; i--)
					{
						stack.Push(current.children[i]);
					}
				}
			}
		}

		public IPLogTreeView (TreeViewState state, MultiColumnHeader multicolumnHeader, TreeModel<LogTreeElement> model) : base (state, multicolumnHeader, model)
		{
			rowHeight                     = kRowHeights;
			columnIndexForTreeFoldouts    = 1;
			showAlternatingRowBackgrounds = true;
			showBorder                    = true;
			customFoldoutYOffset          = (kRowHeights - EditorGUIUtility.singleLineHeight) * 0.5f; // center foldout in the row since we also center content. See RowGUI
			extraSpaceBeforeIconAndLabel = kToggleWidth;
			multicolumnHeader.sortingChanged += OnSortingChanged;
			
			Reload();
		}


        // Note we We only build the visible rows, only the backend has the full tree information. 
        // The treeview only creates info for the row list.
        protected override IList<TreeViewItem> BuildRows(TreeViewItem root)
		{
			var rows = base.BuildRows (root);
			SortIfNeeded (root, rows);
			return rows;
		}

		void OnSortingChanged (MultiColumnHeader multiColumnHeader)
		{
			SortIfNeeded (rootItem, GetRows());
		}

		void SortIfNeeded (TreeViewItem root, IList<TreeViewItem> rows)
		{
			if (rows.Count <= 1)
				return;
			
			if (multiColumnHeader.sortedColumnIndex == -1)
			{
				return; // No column to sort for (just use the order the data are in)
			}
			

			TreeToList(root, rows);
			Repaint();
		}

	
		protected override void RowGUI (RowGUIArgs args)
		{
			var item = (TreeViewItem<LogTreeElement>) args.item;

			for (int i = 0; i < args.GetNumVisibleColumns (); ++i)
			{
				CellGUI(args.GetCellRect(i), item, (MyColumns)args.GetColumn(i), ref args);
			}
		}

		void CellGUI (Rect cellRect, TreeViewItem<LogTreeElement> item, MyColumns column, ref RowGUIArgs args)
		{
			// Center cell rect vertically (makes it easier to place controls, icons etc in the cells)
			CenterRectUsingSingleLineHeight(ref cellRect);

			switch (column)
			{
				case MyColumns.IP:
					{
						// Do toggle
						Rect toggleRect  = cellRect;
						toggleRect.x     += GetContentIndent(item);
						toggleRect.width = kToggleWidth;
					
						EditorGUI.LabelField(toggleRect, item.data.ip); // hide when outside cell rect
						args.rowRect = cellRect;
						base.RowGUI(args);
					}
					break;
				case MyColumns.STATE:
					{
						// Do toggle
						Rect toggleRect  = cellRect;
						toggleRect.x     += GetContentIndent(item);
						toggleRect.width = 60;
						if (toggleRect.xMax < cellRect.xMax)
						{
							if (GUI.Button(toggleRect, item.data.state ? "ON" : "OFF")) // hide when outside cell rect
							{
								item.data.state = !item.data.state;
								if (item.data.state)
								{
									LogCommon.Listaning(item.data.ip);
								}
							}
						}
						base.RowGUI(args);
					}
					break;

			}
		}


		public static MultiColumnHeaderState CreateDefaultMultiColumnHeaderState(float treeViewWidth)
		{
			var columns = new[] 
			{
				new MultiColumnHeaderState.Column 
				{
					headerContent = new GUIContent("IP"),
					headerTextAlignment = TextAlignment.Center,
					width = treeViewWidth/2,
					minWidth = 60,
				},
				new MultiColumnHeaderState.Column 
				{
					headerContent = new GUIContent("State"),
					headerTextAlignment = TextAlignment.Center,
					width = treeViewWidth/2,
					minWidth = 60,
				}
			};
			var state =  new MultiColumnHeaderState(columns);
			return state;
		}
	}
}

#endif
