#if LOGCAT_SERVER

using System;
using System.Collections.Generic;
using UnityEditor.Callbacks;
using UnityEditor.IMGUI.Controls;
using UnityEngine;
using UnityEngine.UIElements;


namespace UnityEditor.TreeViewExamples
{

	class IPLogWindow : EditorWindow
	{
		[NonSerialized] bool m_Initialized;
		[SerializeField] TreeViewState m_TreeViewState; // Serialized in the window layout file so it survives assembly reloading
		[SerializeField] MultiColumnHeaderState m_MultiColumnHeaderState;
		SearchField m_SearchField;
		IPLogTreeView m_TreeView;

		LogcatServer m_logcat;

		List<LogTreeElement> logs => m_logcat.logTrees;

		[MenuItem("Logcat/window")]
		public static IPLogWindow GetWindow ()
		{
			var window = GetWindow<IPLogWindow>();
			window.titleContent = new GUIContent("Logcat");
			window.Focus();
			window.Repaint();
			return window;
		}

		bool isPlaying = false;

		Rect multiColumnViewRect
		{
			get { return new Rect(20, 30, position.width - 40, position.height - 40); }
		}

		Rect toolbarRect
		{
			get { return new Rect(20f, 10f, position.width - 40f, 20f); }
		}

		public IPLogTreeView treeView
		{
			get { return m_TreeView; }
		}

		void InitIfNeeded ()
		{
			if (!m_Initialized)
			{
				// Check if it already exists (deserialized from window layout file or scriptable object)
				if (m_TreeViewState == null)
					m_TreeViewState = new TreeViewState();

				if (m_logcat == null)
				{
					m_logcat = new LogcatServer();
					m_logcat.Init();
				}		
				m_logcat.changed = Changed;

				bool firstInit = m_MultiColumnHeaderState == null;
				var headerState = IPLogTreeView.CreateDefaultMultiColumnHeaderState(multiColumnViewRect.width);
				if (MultiColumnHeaderState.CanOverwriteSerializedFields(m_MultiColumnHeaderState, headerState))
					MultiColumnHeaderState.OverwriteSerializedFields(m_MultiColumnHeaderState, headerState);
				m_MultiColumnHeaderState = headerState;
				
				var multiColumnHeader = new MultiColumnHeader(headerState);
				if (firstInit)
					multiColumnHeader.ResizeToFit ();

				multiColumnHeader.canSort = false;

			    var treeModel = new TreeModel<LogTreeElement>(GetData());
				
				m_TreeView = new IPLogTreeView(m_TreeViewState, multiColumnHeader, treeModel);

				m_SearchField = new SearchField();
				m_SearchField.downOrUpArrowKeyPressed += m_TreeView.SetFocusAndEnsureSelectedItem;

			

				m_Initialized = true;
			}
		}


		/// <summary>
		/// This function is called when the MonoBehaviour will be destroyed.
		/// </summary>
		void OnDestroy()
		{
			m_logcat.Release();
		}

		IList<LogTreeElement> GetData()
		{
			m_logcat.ClearElement();
			m_logcat.AddElement(new LogTreeElement("Root", -1, logs.Count));
			// generate some test data
			return logs;
		}


		public void Changed()
		{
			m_TreeView.treeModel.SetData(logs);
			m_TreeView.Reload();
		}

		public void Clear()
		{
			m_logcat.ClearElement();
			m_logcat.AddElement(new LogTreeElement("Root", -1, logs.Count));
			m_TreeView.treeModel.SetData(logs);
			m_TreeView.Reload();
		}

		void OnGUI ()
		{
			InitIfNeeded();

			SearchBar (toolbarRect);

			GUI.enabled = isPlaying;
			DoTreeView (multiColumnViewRect);
			GUI.enabled = true;

		}

		void SearchBar (Rect rect)
		{
			rect.width -= rect.width / 2;
			rect.y -= 2;
			BottomToolBar(rect);

			rect.x += rect.width;
			rect.y += 2;
			treeView.searchString = m_SearchField.OnGUI(rect, treeView.searchString);
		}

		void DoTreeView (Rect rect)
		{
			m_TreeView.OnGUI(rect);
		}

		void BottomToolBar (Rect rect)
		{
			GUILayout.BeginArea (rect);

			using (new EditorGUILayout.HorizontalScope())
			{
				if (GUILayout.Button(EditorGUIUtility.IconContent(PlayIcon(isPlaying)), GUILayout.MaxWidth(80)))
				{
					isPlaying = !isPlaying;
				}

				if (GUILayout.Button("Clear All"))
				{
					Clear();
				}

				GUILayout.FlexibleSpace();
			}
			GUILayout.EndArea();
		}

		string PlayIcon(bool value)
		{
			return value ? "d_preAudioPlayOn" : "d_preAudioPlayOff";
		}
	}
}
#endif
